/* 11.24.2001: automatic eCS update procedure with support for the           */
/*             fixpack 1 (xr_c001.zip) and ecsupdate (eCSUpdt.zip)           */
/* 11.30.2001: added support for kernel updates                              */
/* 12.01.2001: added postprocessing steps (-39 warnings)                     */
/* 12.04.2001: video.zip was not updated                                     */
/* 12.10.2001: tcpip.zip was not updated                                     */

/* read command line parameters */
parse arg cddrive

/* load configuration file */
call read_config_file 'updcd.cfg'

/* check cddrive value */
if substr(cddrive, 2) = ':' then w4cd = translate(cddrive)

/* make sure they exist */
'@mkdir 'burn' >nul 2>>&1'
'@mkdir 'fixes' >nul 2>>&1'
'@mkdir 'updates' >nul 2>>&1'
'@mkdir fix >nul 2>>&1'

/* updcd version */
ver = 1.99
n.errors = 0
n.warnings = 0
n.elapsed = time('E')

/* load rexxutil functions we need */
call rxfuncadd sysfiletree, rexxutil, sysfiletree
call rxfuncadd syscurstate, rexxutil, syscurstate

/* remember we were started */
'@echo go4it busy > go4it.bsy'

/* clean up things */
'@mode 80,25 >nul'
'@ansi on >NUL'
say '1B'x || '[36;44m'
say '1B'x || '[1m'
'@cls'
say
say ' Searching for components which can be integrated. Please wait!'
call syscurstate off

/* find fixpaks located in the subdirectory fixes */
say
startdir = fixes
fpk. = ''
rc = lineout(, " Fixpak:    Type:     Directory:")
rc = lineout(, " -------------------------------")
rc = SysFileTree(startdir'\xr?c*.zip', ini., 'FSO') 
if ini.0 > 0 then do
	ini.1 = translate(ini.1)
	fpk.cp1.fnd = 1
	fpk.cp1.typ = substr(ini.1, pos('XR', ini.1), 7)
	fpk.cp1.num = substr(fpk.cp1.typ, 5)
	fpk.cp1.dir = translate(substr(ini.1, 1, lastpos('\', ini.1)-1))
	rc = lineout(, " "fpk.cp1.typ"    CP_1      "fpk.cp1.dir)
	fpk.cp1.typ = 'XR'substr(fpk.cp1.typ, 4)
end
else call lineout "", " Found nothing..."

/* find updated products located in the subdirectory updates */
say
startdir = updates
upd. = ''
rc = lineout(, " Update:    Type:     Directory:")
rc = lineout(, " -------------------------------")
rc = SysFileTree(startdir'\eCSUpdt.cmd', ini., 'FSO') 

if ini.0 > 0 then do
	upd.ec1.fnd = 1
	upd.ec1.typ = 'UPDTOOL'
	upd.ec1.num = substr(upd.ec1.typ, 4)
	upd.ec1.dir = translate(substr(ini.1, 1, lastpos('\', ini.1)-1))
	rc = lineout(, " "upd.ec1.typ"    EC_1      "upd.ec1.dir)
end
else call lineout "", " Found nothing..."

/* determining language to use */
ini = w4cd'\os2\install\syslevel.os2'
if stream(ini, 'c', 'query exists') <> '' then do
	do while lines(ini)
			l=linein(ini)
			if pos('Convenience Package', l) > 0 then do
				lang = substr(l, pos('XR', l), 3)
				/* say lang */
			end
	end
	rc = lineout(ini)
end
else do
	Say
	Say ' No CD-ROM found! Exiting...'
	rc = lineout(log, 'Error: No CD-ROM found! Exiting...')
	'@del go4it.bsy >nul 2>>&1'
	exit 9
end

select
	when lang = 'XR0' then lang = 'EN (English)'
	when lang = 'XRG' then lang = 'DE (German)'
	when lang = 'XRN' then lang = 'NO (Norwegian)'
	when lang = 'XRV' then lang = 'TW (Thai)'
	when lang = 'XRF' then lang = 'FR (French)'
	when lang = 'XRI' then lang = 'IT (Italian)'
	when lang = 'XRJ' then lang = 'JP (Japanese)'
	when lang = 'XRK' then lang = 'KO (Korean)'
	when lang = 'XRB' then lang = 'BR (Brazilian)'
	when lang = 'XRC' then lang = 'CN (Chinese)'
	when lang = 'XRS' then lang = 'ES (Spanish)'
	when lang = 'XRD' then lang = 'DK (Danish)'
	when lang = 'XRL' then lang = 'FI (Finnish)'
	when lang = 'XRH' then lang = 'NL (Dutch)'
	when lang = 'XRY' then lang = 'HU (Hungarian)'
	when lang = 'XRR' then lang = 'RU (Russian)'
  when lang = 'XRA' then lang = 'AR (Arabic)'
	otherwise lang = 'Unknown (assuming English for universal products)'
end
say
rc = lineout(, " Detected language: "lang)

say
say ' Press any key to start the build process or CTRL-C to abort'
'@pause >nul'

/* switch off deldir if set */
ddir = value("DELDIR", , "OS2ENVIRONMENT")
if length(ddir) > 0 then do
	rc = lineout(log, 'Info: Switching off deldir...')
	ddir = value("DELDIR", "", "OS2ENVIRONMENT")
end

'@del 'log' >nul 2>>&1'
rc = lineout(log, 'Info: Searching for fixes and updates...')
rc = lineout(log, 'Info: Found fix 'fpk.cp1.typ' CP_1 'fpk.cp1.dir)
rc = lineout(log, 'Info: Found update 'upd.ec1.typ' UPDTOOL 'upd.ec1.dir)
rc = lineout(log, 'Info: Detected language: 'lang)
rc = lineout(log)
parse var lang w1 w2
lang = space(w1)
if lang = 'HU' | lang = 'RU' | lang = 'AR' | lang = 'Unknown' then lang = 'EN'

/* check if we have at least the base fixes */
if fpk.cp1.dir = '' & upd.ec1.dir = '' then do
	Say
	Say ' The obligatory components should be present! Exiting...'
	rc = lineout(log, 'Error: The obligatory components should be present! Exiting...')
	'@del go4it.bsy >nul 2>>&1'
	exit 9
end

/* determine eCS base (MCP/ACP) */
/* do not know how to do this */
/* let us assume we have MCP */
cptype = 'MCP'
rc = lineout(log, 'Info: Detected CP: 'cptype)
rc = lineout(log)

/**********************/
/* Clean up old image */
/**********************/

'@cls'
say '1B'x || '[2;2HUpdate eCS installation CD-ROM, version 'ver
say '1B'x || '[4;2HTarget directory      : 'burn
say '1B'x || '[5;2HSource directory      : 'burn
say '1B'x || '[7;2HCurrent activity      : 'substr('Cleaning up directory: 'burn, 1, 50)
say '1B'x || '[8;2HTarget object         : 'substr('Searching...', 1, 50)
say '1B'x || '[9;2HSource object         : 'substr('Searching...', 1, 50)
say '1B'x || '[11;2HUpdate action         : 'substr('Searching...', 1, 50)
say '1B'x || '[12;2HUpdate return code    : 0'
say '1B'x || '[14;2HElapsed time [m]      : 'trunc(time('E')/60 + n.elapsed, 2)
say '1B'x || '[15;2HNumber of errors      : 'n.errors
say '1B'x || '[16;2HNumber of warnings    : 'n.warnings
say '1B'x || '[17;2HUpdate log            : 'log
say '1B'x || '[19;2H'

/* clean up burn */
plugin = 'cleanup.pin'
do while lines(plugin)
	l = linein(plugin)
	interpret l
end
call lineout plugin

/* clean up fix */
rc = SysFileTree('fix\*', ini., 'FSO',,'**---')  
do i = 1 to ini.0
	call execute '@del 'ini.i
end

/* remove RO attributes */
call execute '@attrib -r -s -h -a * /s'
call execute '@attrib -r -s -h -a 'fixes'\* /s'
call execute '@attrib -r -s -h -a 'updates'\* /s'

/* create some directories */
'@mkdir 'burn '>nul 2>&1'
call execute '@mkdir 'burn'\updcd'
call execute '@mkdir 'burn'\updcd\bootimg'
call execute '@mkdir 'burn'\updcd\bootimg\DISK_0_1.IMG'
call execute '@mkdir 'burn'\updcd\bootimg\DISK_2.IMG'

/* copy CD-ROM */
say '1B'x || '[4;26H'substr(burn, 1, 50)
say '1B'x || '[5;26H'substr(w4cd, 1, 50)
say '1B'x || '[7;26H'substr('Copying CD-ROM files', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
call execute '@xcopy 'w4cd'\* 'burn'\. /H/O/T/S/R/V/E'

/* add fixpak files */
say '1B'x || '[7;26H'substr('Copying base CP fixes', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
/* ACP? */
call execute '@unzip -oj 'fpk.cp1.dir'\xr?c*.zip -d fix'

/* delete OREXX1.DL_ otherwise we will not have classic REXX support */
if stream('fix\OREXX1.DL_',  'c', 'query exists') <> '' then call execute '@del fix\OREXX1.DL_'

/* remove more */
call execute '@del fix\readme.cid'
call execute '@del fix\readme.1st'
call execute '@del fix\readme2'
call execute '@del fix\csf_disk'
call execute '@del fix\srv_prod.os2'
call execute '@del fix\README.DBG'
/* only for MCP? */
if cptype = 'MCP' then do
	call execute '@del fix\jfs.if_'
	call execute '@del fix\cachejfs.ex_'
	call execute '@del fix\jfs.td_'
	call execute '@del fix\trc012f.tf_'
	call execute '@del fix\cchmain.ex_'
	call execute '@del fix\EZPLAY2.DL_'
	call execute '@del fix\GSVINST.EX_'
	call execute '@del fix\PMPRNTXT.QP_'
	call execute '@del fix\PSSDMON.EX_'
	call execute '@del fix\RJAPPLET.EX_'
	call execute '@del fix\WELCOME.EX_'
	call execute '@del fix\WPNLS.DL_'
end

/* prepare it */
'@del result.out >nul 2>&1'
'@call prpfp fix 'n.errors n.warnings trunc(time('E')/60 + n.elapsed, 2) ver log
res = linein('result.out')
rc = lineout('result.out')
parse var res n.errors n.warnings

/* add npconfig fix */
if cptype = 'MCP' then call execute '@unzip -oj 'fpk.cp1.dir'\mcpc*.zip -d fix'

/* add kernel update */
call execute '@unzip -oj 'fpk.cp1.dir'\w4*.zip  -d fix'
if stream('fix\readme', 'c', 'query exists') <> '' then call execute '@del fix\readme'

/* seems we need REXXSOM.DLL */
/* call execute '@copy 'burn'\OS2\dll\orexxsom.dll fix\rexxsom.dll' */

/* add base addons */
call SysFileTree fpk.cp1.dir'\addons\*', 'ini.', 'FO'
if ini.0 > 0 then call execute '@xcopy 'fpk.cp1.dir'\addons\* fix\.'
if stream('fix\readme', 'c', 'query exists') <> '' then call execute '@del fix\readme'
if cptype = 'ACP' then do
	call SysFileTree fpk.cp1.dir'\addons\smp\*', 'ini.', 'FO'
	if ini.0 > 0 then call execute '@xcopy 'fpk.cp1.dir'\addons\smp\* fix\smp\.'
	if stream('fix\smp\readme', 'c', 'query exists') <> '' then call execute '@del fix\smp\readme'
end

/*************************************/
/* Update installation floppy images */
/*************************************/

/* finally run update */
say '1B'x || '[7;26H'substr('Updating installation floppy images', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
'@del result.out >nul'
'@call updif2 'burn' fix 'n.errors n.warnings trunc(time('E')/60 + n.elapsed, 2) log ver expert add_dani add_dummy upd.fd1.dir upd.fd2.dir 
res = linein('result.out')
rc = lineout('result.out')
parse var res n.errors n.warnings 

/* remove RO attribute */
rc = SysFileTree(burn'\*', ini., 'BSO',,'**---')  
call execute '@attrib -r -s -h -a 'burn'\* /s'

/********************************************************************/
/* incorporate CP1 fixpak                                           */
/********************************************************************/

say '1B'x || '[7;26H'substr('Updating CP files', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)

/* reorganise pmdf */
call execute '@mkdir 'burn'\os2\pdpsi\pmdf\warp45'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\DF_DEB.EXE  	'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\DF_RET.EXE  	'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\KERNEL.SDF  	'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\KERNELD.SDF  'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\PMCTLS.SYM   'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\PMMERGE.SYM  'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\PMSPL.SYM   	'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\PMVIOP.SYM  	'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\PMWP.SYM   	'burn'\os2\pdpsi\pmdf\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\DF_DEB.EXE   'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\DF_RET.EXE   'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\DF_DEB.EXE   'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\DOSCALL1.SYM 'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\KERNEL.SDF   'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\KERNELD.SDF  'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\OS2KRNLB.SYM 'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\OS2KRNLD.SYM 'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\OS2KRNLR.SYM 'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@copy 'burn'\os2\pdpsi\pmdf\warp45_u\SHELL.SDF    'burn'\os2\pdpsi\pmdf\warp45\.'
call execute '@mkdir 'burn'\tmppdpsi'
call execute '@xcopy 'burn'\OS2\PDPSI\PMDF\WARP45_U\* 'burn'\tmppdpsi\.'
call deldir burn'\OS2\PDPSI\PMDF\WARP45_U'

/* add tedit help */
call execute '@unzip -oj 'upd.ec1.dir'\tedit.zip -d fix'

/* copy ecsguide to os2 to get updated */
call execute '@copy 'burn'\ecs\ecsguide.zip 'burn'\os2\.'

/* run update */
'@del result.out >nul'
'@copy basefpk.ec1 basefpk.pin >nul'
'@call updcd 'burn' fix 'n.errors n.warnings trunc(time('E')/60 + n.elapsed, 2) log ver
res = linein('result.out')
rc = lineout('result.out')
parse var res n.errors n.warnings 

/* postprocessing */
call execute '@copy 'burn'\updcd\bootimg\DISK_0_1.IMG\os2ldr 'burn'\ecs\disk01\.'
call execute '@mkdir 'burn'\OS2\PDPSI\PMDF\WARP45_U'
call execute '@xcopy 'burn'\tmppdpsi\* 'burn'\OS2\PDPSI\PMDF\WARP45_U\.'
call deldir burn'\tmppdpsi'
call execute '@copy 'burn'\os2\ecsguide.zip 'burn'\ecs\.'
call execute '@del 'burn'\os2\ecsguide.zip'

/* and delete temp stuff from fix */
rc = SysFileTree('fix\*', ini., 'FSO',,'**---')  
do i = 1 to ini.0
	call execute '@del 'ini.i
end
if cptype = 'ACP' then call execute '@rmdir fix\smp'

if expert = 0 then do
	say '1B'x || '[19;2HPress any key to continue or CTRL-C to abort'
	'@pause >nul'
	say '1B'x || '[19;2H                                            '
end

/********************************************************************/
/* Dialog Enhancer and Theme Manager fixes                          */
/********************************************************************/

/* add dialog enhancer modules */
say '1B'x || '[7;26H'substr('Updating Dialog Enhancer files', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
call execute '@unzip -oj 'fpk.cp1.dir'\dlgc*.zip -d fix'
call execute '@unzip -oj 'upd.ec1.dir'\dlgc*.zip -d fix'
call execute '@del fix\pmsdmri.cmd'
call execute '@copy 'burn'\os2\dll\pmmrgres.dll fix\.'

/* create theme manager modules */
say '1B'x || '[7;26H'substr('Updating Theme Manager files', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
call execute '@unzip -oj 'fpk.cp1.dir'\thmcode.zip -d fix'
bmp.0 = 15
bmp.1 = 25
bmp.2 = 26
bmp.3 = 27
bmp.4 = 28
bmp.5 = 29
bmp.6 = 30
bmp.7 = 31
bmp.8 = 32
bmp.9 = 33
bmp.10 = 34
bmp.11 = 48
bmp.12 = 50
bmp.13 = 51
bmp.14 = 52
bmp.15 = 53

cdir = chdir()
call chdir fix
Do i = 1 To bmp.0
   Call Resmgrn '-d pmmrgres.dll' bmp.i || '.bitmap'
End

res.0 = 11
res.1 = ecs
res.2 = ecsmath
res.3 = ecswarp
res.4 = ecswarp2
res.5 = ecswin
res.6 = flat
res.7 = warp3
res.8 = warp4
res.9 = warpal
res.10 = win
res.11 = warpslv

call execute '@xcopy 'burn'\ecs\thememgr\*.res .'
call execute '@xcopy 'burn'\ecs\thememgr\*.bmp .'
Do i = 1 To res.0
   call execute '@copy pmmrgres.dll' res.i || '.dll'
   Call Resmgrn '-a' res.i || '.dll' res.i || '.res'
End
call execute '@copy ecs.dll 'burn'\os2\dll\pmmrgres.dll'

call execute '@del *.res'
call execute '@del *.bmp'
call execute '@del resmgrn.cmd'
call execute '@del themeupd.cmd'
call execute '@del rdcn.cmd'
call execute '@del RC.EXE'
call execute '@del RCPP.EXE'
call execute '@del RCPP.ERR'
call execute '@del rdcpp.exe'
call chdir cdir

/* put zips to temp dir which should be updated */
call execute '@mkdir 'burn'\ziptemp'
call execute '@copy 'burn'\zipfiles\mptn.zip     'burn'\ziptemp\.'
call execute '@copy 'burn'\zipfiles\video.zip    'burn'\ziptemp\.'
call execute '@copy 'burn'\zipfiles\tcpip.zip    'burn'\ziptemp\.'
/* call execute '@copy 'burn'\zipfiles\printers.zip 'burn'\ziptemp\.' */

/* run update */
'@del result.out >nul'
'@copy basefpk.ec2 basefpk.pin >nul'
'@call updcd 'burn' fix 'n.errors n.warnings trunc(time('E')/60 + n.elapsed, 2) log ver
res = linein('result.out')
rc = lineout('result.out')
parse var res n.errors n.warnings 

/* do some extra postprocessing */
call execute '@copy 'upd.ec1.dir'\dlgc001.dat 'burn'\ecs\install\dlg_inst.dat'

/* put back zips */
call execute '@copy 'burn'\ziptemp\mptn.zip     'burn'\zipfiles\.'
call execute '@copy 'burn'\ziptemp\video.zip    'burn'\zipfiles\.'
call execute '@copy 'burn'\ziptemp\tcpip.zip    'burn'\zipfiles\.'
/* call execute '@copy 'burn'\ziptemp\printers.zip 'burn'\zipfiles\.' */
call deldir burn'\ziptemp'

/* update syslevel */
call update_ecs_syslevel burn'\os2\install\syslevel.os2'

/* and delete temp stuff from fix */
rc = SysFileTree('fix\*', ini., 'FSO',,'**---')  
do i = 1 to ini.0
	call execute '@del 'ini.i
end
if cptype = 'ACP' then call execute '@rmdir fix\smp'

if expert = 0 then do
	say '1B'x || '[19;2HPress any key to continue or CTRL-C to abort'
	'@pause >nul'
	say '1B'x || '[19;2H                                            '
end

/************************/
/* add UPDCD info to CD */
/************************/

call execute '@mkdir 'burn'\updcd\bin'
call execute '@copy *.cm? 'burn'\updcd\bin\.'
call execute '@copy *.exe 'burn'\updcd\bin\.'
call execute '@copy *.dll 'burn'\updcd\bin\.'
call execute '@copy *.msg 'burn'\updcd\bin\.'
call execute '@copy *.cfg 'burn'\updcd\bin\.'
call execute '@copy cd2boot   'burn'\updcd\bin\.'
call execute '@copy pmfaxlt.* 'burn'\updcd\bin\.'
call execute '@copy basefpk.* 'burn'\updcd\bin\.'
call execute '@copy mptsfpk.* 'burn'\updcd\bin\.'
call execute '@copy tcppfpk.* 'burn'\updcd\bin\.'
call execute '@copy peerfpk.* 'burn'\updcd\bin\.'
call execute '@mkdir 'burn'\updcd\doc'
call execute '@copy *.txt 'burn'\updcd\doc\.'
call execute '@copy readme.* 'burn'\updcd\doc\.'
call execute '@mkdir 'burn'\updcd\log'
rc = lineout(log, 'Info: Total number of errors  : 'n.errors)
rc = lineout(log, 'Info: Total number of warnings: 'n.warnings)
rc = lineout(log)
'@copy *.log 'burn'\updcd\log\. >nul 2>>&1'

/***********/
/* burn CD */
/***********/
cdrprg = "record.cmd"
if stream(cdrprg, 'c', 'query exists') <> '' then do
	say '1B'x || '[7;26H'substr('Calling script 'cdrprg' to burn CD.', 1, 50)
	say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
	say '1B'x || '[19;2H'
	'@call 'cdrprg n.errors n.warnings
end
else say "Cannot find burning script: "cdrprg

/***********************/
/* clean up and report */
/***********************/
'@del result.out >nul'

/* signal fatal errors */
if n.errors > 0 then 	say '1B'x || '[19;2HThere were one or more errors detected, Check 'log'!'
else say '1B'x || '[19;2HUpdate is completed succesfully!'

'@del go4it.bsy >nul 2>>&1'
exit


/* delete directory */
DelDir: procedure expose n. log

	parse upper arg Directory
	DirSpec = Directory || '\*'

	/* delete subdirectories */
	rc = SysFileTree(DirSpec, Subdirs, 'DO', '*****', '-*---')
	do i = 1 to Subdirs.0
   		call DelDir Subdirs.i
	end

	/* delete files */
	rc = SysFileTree(DirSpec, Files, 'FO', '*****', '-*---')
	do i = 1 to Files.0
      call execute '@del "'Files.i'"'
  end

	/* delete directory */
	cdir1 = chdir()
	cdir2 = chdir(Directory)
	call chdir '\'
	cdir1 = chdir(cdir1)
	if cdir2 = Directory then call execute '@rmdir "'Directory'"'

return

/* execute command, display it and write to log */ 
execute: procedure expose log n. iddir

	parse arg comm 
	rcode = 0 /* heck */
	if pos('"', comm) > 0 then do
		parse upper var comm comm1 comm2
		comm3 = ''
		comm4 = ''
	end
	else 
		parse upper var comm comm1 comm2 comm3 comm4 rest
	/* do not copy or pack if file does not exist */
	comm2 = strip(comm2, 'B', '"')
/*'@echo 'comm2' >>'log*/
	rc = SysFileTree(comm2, 'tmp.', 'FSO') 
	if (comm1 <> '@COPY' & comm1 <> '@PACK2' & comm1 <> '@DEL' & comm1 <> '@XCOPY') | (stream(comm2, 'c', 'query exists') <> '') | (tmp.0 > 0) then do
		comm' >nul 2>>&1'
		if rc <> 0 then rcode = rc
	end
	else do /* log warning */
		if comm1 <> '@DEL' then do
			rc = lineout(log, 'Warning: 'comm2' does not exist!')
			n.warnings = n.warnings + 1
			call report_status
		end
		return
	end
	comm = comm ||' rc='rcode

	if comm1 = '@..\ZIP.EXE' then do	
		say '1B'x || '[8;26H'substr(space(reverse(substr(reverse(comm3), 1, 50))), 1, 50)
		say '1B'x || '[9;26H'substr(space(reverse(substr(reverse(comm4), 1, 50))), 1, 50)
		say '1B'x || '[11;26HZip         '
	end
	else if comm1 = '@MKDIR' | comm1 = '@RMDIR' | comm1 = '@CD' then do	
		nop;
	end
	else if comm1 = '@UNZIP' then do	
		say '1B'x || '[8;26H'substr(space(reverse(substr(reverse(rest), 1, 50))), 1, 50)
		say '1B'x || '[9;26H'substr(space(reverse(substr(reverse(comm3), 1, 50))), 1, 50)
		say '1B'x || '[11;26HUnzip         '
	end
	else do 
		say '1B'x || '[8;26H'substr(space(reverse(substr(reverse(comm3), 1, 50))), 1, 50)
		say '1B'x || '[9;26H'substr(space(reverse(substr(reverse(comm2), 1, 50))), 1, 50)
		say '1B'x || '[11;26H'substr(comm1, 2, 50)
	end
	say '1B'x || '[12;26H'rcode' '

	if rcode <> 0 then do
		rc = lineout(log, 'Error: 'comm)
		rc = lineout(log)
		n.errors = n.errors + 1
	end
	else do
		rc = lineout(log, 'Info: 'comm)
		rc = lineout(log)
	end

	call report_status

return rcode

/* display status */
report_status: procedure expose n. 

	say '1B'x || '[14;2HElapsed time [m]      : 'trunc(time('E')/60 + n.elapsed, 2)
	say '1B'x || '[15;2HNumber of errors      : 'n.errors
	say '1B'x || '[16;2HNumber of warnings    : 'n.warnings
	say '1B'x || '[19;2H'

return

/* copy files from sdir to tdir except ea data. sf */
CopyDir: procedure expose log n.

	parse arg sdir tdir 

	call SysFileTree sdir'\*', ini., 'FO', , '**---'
	do i=1 to ini.0
		if pos('EA DATA. SF', translate(ini.i)) = 0 then do
			'@copy 'ini.i tdir'\. >nul 2>>&1'
			rcode = rc
			if rcode <> 0 then do
				rc = lineout(log, 'Error: @copy 'ini.i tdir'\. (rc='rcode')')
				rc = lineout(log)
				n.errors = n.errors + 1
			end
			else do
				rc = lineout(log, 'Info: @copy 'ini.i tdir'\. (rc='rcode')')
				rc = lineout(log)
			end
		end
	end
	call report_status

return

/* read the updcd cfg file and get absolute directory path */
Read_Config_File: 

	parse arg cfgfile

	/* check */
	if stream(cfgfile, 'c', 'query exists') = '' then do
		say 'Fatal error! The configuration file ('cfgfile') is missing.'
		say 'Please run INSTALL.CMD to create one.'
		'@pause'
		exit 9
	end

	/* read file */
	do while lines(cfgfile)
		l = linein(cfgfile)
		interpret l
	end
	call lineout cfgfile 

	/* need to know abolute path */
	cdir    = chdir()
	burn    = chdir(burn)
	cdir    = chdir(cdir)
	w4cd		= chdir(w4cd)
	cdir    = chdir(cdir)
	fixes   = chdir(fixes)
	cdir    = chdir(cdir)
	updates = chdir(updates)
	cdir    = chdir(cdir)
	addons  = chdir(addons)
	cdir    = chdir(cdir)
	rootdir = chdir(rootdir)
	cdir    = chdir(cdir)
	tempdir = chdir(tempdir)
	cdir    = chdir(cdir)
	source	= chdir(source)
	cdir    = chdir(cdir)
	cdrdir	= chdir(cdrdir)
	cdir    = chdir(cdir)

	/* need to know abolute path for files */
	if filespec('drive', log) = '' then 
		log	= cdir||'\'||filespec('name', log)
	if filespec('drive', tempraw) = '' then 
		tempraw	= cdir||'\'||filespec('name', tempraw)
	if filespec('drive', burnlog) = '' then 
		burnlog	= cdir||'\'||filespec('name', burnlog)

return

/* chdir: drive and path aware directory() replacement       */
/*        returns the current dir without trailing backslash */
chdir: procedure
	parse upper arg dr                /* get target dir        */
	if length(dr) > 1 then dr = strip(dr, 'T', '\')
	cdir = directory()                /* get current dir       */
	if substr(dr, 2, 1) = ':' then do /* absolute path         */
		drive = substr(dr, 1, 2)
		rc = directory(drive)           /* change to drive       */
		if substr(rc, 1, 2) <> drive then 
			return strip(directory(cdir), 'T', '\')
		rc = directory('\')             /* change to root        */
		rc = directory(substr(dr, 3))   /* change to path        */
		if strip(rc, 'T', '\') <> dr then 
			return strip(directory(cdir), 'T', '\')
	end
	else do
		return strip(directory(dr), 'T', '\')  /* relative path  */
	end
return strip(directory(), 'T', '\') /* return current dir    */

/* updat ecs syslevel file */
update_ecs_syslevel: procedure expose n. log

	parse arg sys_lvl

	sys_lvl_length = Stream(sys_lvl, 'c', 'query size')
	Call Stream sys_lvl, 'c', 'open read'
	sys_lvl_org = Charin(sys_lvl, 1, sys_lvl_length)
	Call Stream sys_lvl, 'c', 'close'
	call execute '@del 'sys_lvl

	Call Stream sys_lvl, 'c', 'open write'
	rc = Charout(sys_lvl, Left(sys_lvl_org, 165))
	Call Stream sys_lvl, 'c', 'close'

	sys_lvl_bkp = substr(sys_lvl, 1, pos('.OS2', translate(sys_lvl)))||'ORG'
	call execute '@copy 'sys_lvl sys_lvl_bkp
	sys_lvl_length = Stream(sys_lvl, 'c', 'query size')
	sys_lvl_org = Charin(sys_lvl, 1, sys_lvl_length)
	Call Stream sys_lvl, 'c', 'close'
	upd_desc = 'eComStation Operating System'
	sys_lvl_new = Left(sys_lvl_org, 60) || Left(upd_desc, 80, '00'x)
	sys_lvl_new = sys_lvl_new || Substr(sys_lvl_org, 141)
	call execute '@del 'sys_lvl

	Call Stream sys_lvl, 'c', 'open write'
	rc = Charout(sys_lvl, sys_lvl_new, 1)
	Call Stream sys_lvl_upd, 'c', 'close'

return
