/* Burn bootable CD-ROM using CDRECORD/2 or RSJ                              */
/*                                                                           */
/* NOTE: This script uses Albert Shan's shareware SVDISK or the freeware     */
/*       VFDISK to circumvent having to use a 2.88MB floppy for the boot     */
/*       img creation process.                                               */
/*                                                                           */
/*       Special thank to Fred Leggett, Jeffrey Smick and Ken Bates.         */
/*                                                                           */
/* 11.25.2000: added code of Fred Leggett to automate the boot img creation  */
/* 12.13.2000: addes some code to align with the PM interface                */
/* 02.19.2001: fixed problem with label disk 2                               */
/* 02.27.2001: added new menupoint (6)                                       */
/* 03.04.2001: fixed problem with flag file                                  */
/* 03.19.2001: some cosmethic changes                                        */
/* 04.10.2001: solved incomplete do end bug                                  */
/* 05.10.2001: added support for VFDISK 2.0 or higher                        */
/* 05.19.2001: added more logging                                            */
/* 06.09.2001: bug fixing, small improvements                                */
/* 10.27.2001: fixed problem with active drive (@cd -> call directory)       */
/* 10.28.2001: fixed another problem with active drive (cdboot.exe)          */
/* 11.11.2001: minor textual changes, added check of image size              */
/* 11.16.2001: merged with rsjrecrd.cmd, added chdir function, bug fixing    */ 
/* 11.17.2001: fixed extrap which was not working                            */
/* 12.10.2001: changed for ecs                                               */

/* get command line parameters */
parse arg n.errors n.warnings
if n.errors   = '' then n.errors   = '?'
if n.warnings = '' then n.warnings = '?'

/* set variable defaults */
attached 	= 0	/* attached status flag for cd burner 		*/
CD 		= ''	/* letter to attach cd burner to (default Z:)	*/
automated 	= 0	/* users batch job flag 			*/
In_Batch_Mode 	= 'NO'
response 	= 'NONE'

/* load rexxutil */
if RxFuncQuery('SysLoadFuncs') then do
	call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
	call SysLoadFuncs
end

/* read paramaters from updcd.cfg */
call read_config_file 'updcd.cfg'

/* Find out if user has RSJ cdwriter system installed 		*/
/* do this by checking the class list for CDWRITER class 	*/
rsjdir = ''
call SysQueryClassList 'junk.'
i = 1
found = 0
do while ((i <= junk.0) & (\found))
  tmp = translate(word(junk.i, 1))
  if (tmp == 'CDWRITER') then
    do
     /* We found the rsj CDWRITER class */
      tmp = word(junk.i, 2)
      rsjdir = substr(tmp, 1, lastpos('\', tmp )-1)
      found = 1
    end
  i = i + 1
end

/* check vdisk type */
vdisk = SysIni(, 'UPDCD', 'VDISK')
if (vdisk == 'ERROR:') then do
	vdisk = 'SVDISK' /* let us make SVDISK the default */
	call SysIni , 'UPDCD', 'VDISK', vdisk
end

/* check burning program */
bprog = SysIni(, 'UPDCD', 'BPROG')
if bprog = "ERROR:" then do
	bprog = "CDR"
	call SysIni , 'UPDCD', 'BPROG', bprog
end

/* ensure we are in updcd directory. */
call chdir rootdir 

/* remember we were started */
'@echo record.cmd busy > record.bsy'

/* reset more variables */
boot 	= ''
key 	= ''
flag1 	= ' '
flag2 	= ' '
flag3 	= ' '
flag4 	= ' '
flag5 	= ' '
flag6 	= ' '
'@mode 80,25 >nul'
'@ansi on >NUL'
say '1B'x || '[36;44m'
say '1B'x || '[1m'
'@cls'

/* start menu */
text.1 = 'Update CD - Imagefile & CD Burn Script'
text.1 = overlay('Update CD - Imagefile & CD Burn Script', '  :                                                                         :', trunc((74-length(text.1))/2))
text.2 = 'The build procedure has ended with 'n.errors' errors and 'n.warnings' warnings.'
text.2 = overlay('The build procedure has ended with 'n.errors' errors and 'n.warnings' warnings.', '  :                                                                         :', trunc((74-length(text.2))/2))
text.3 = 'See ['log'] for output details.'
text.3 = overlay('See ['log'] for output details.', '  :                                                                         :', trunc((74-length(text.3))/2))

do while (key \= 0)

  '@cls'
  say '  ---------------------------------------------------------------------------'
  say text.1
  say '  ---------------------------------------------------------------------------'
  say text.2
  say text.3
  say '  ---------------------------------------------------------------------------'
  say '  :                                                                         :'
  say '  :  * = Process done.                                                      :'
  say '  :                                                                         :'
  say '  :  0 - Quit.                                                              :'
  say '  :                                                                         :'
  say '  : 'flag1'1 - View burn log.                                                     :'
  say '  : 'flag3'3 - Make CD imagefile.                                                 :'
  say '  : 'flag4'4 - Blank CD-RW (necessary prior to burning if using CD-RW media).     :'
  say '  : 'flag5'5 - Burn CD-R or CD-RW.                                                :'
  say '  :                                                                         :'
  say '  : 'flag6'6 - Let me specify my steps that should run as a batch job.            :'
  say '  :                                                                         :'
  say '  ---------------------------------------------------------------------------'
  say

  if (automated == 0) then do
      key = ''
      do while ((key < 0) | (key = 2) | (key > 6))
        key = SysGetKey('NOECHO')
      end
      say
  end

  do while length(response) > 0
    if (automated == 1) then do
        parse var response job response
        if ((datatype(job) <> 'NUM') | (job < 1) | (job > 5)) then do
		say 'Invalid response, skipping step: 'job
		call Hit_Key 'C'
          	iterate
        end
        else key = job
    end
          
    select
      when (key == 1) then call view_log
      when (key == 2) then call make_bootimage   In_Batch_Mode
      when (key == 3) then call make_cdimage     In_Batch_Mode
      when (key == 4) then call blank_cd         In_Batch_Mode
      when (key == 5) then call burn_cd          In_Batch_Mode
      when (key == 6) then call get_response
    otherwise nop;
    end
    if (automated == 0) then response = '' /* Exit the batch response loop */
  end
  automated = 0
  In_Batch_Mode = 'NO'
  response = 'NONE'
end

/* we are ready */
say 'You may re-run this script by typing RECORD.CMD from 'rootdir
say
say 'Bye bye!'
say
call chdir rootdir /* ensure we are in updcd directory. */
'@del record.bsy >nul 2>>&1'

exit

/* A simple Y/N keypress procedure */
GetKeyYN: procedure

	do while ((key1 <> 'Y') & (key1 <> 'N'))
		key1 = SysGetKey('NOECHO')
		parse upper var key1 key1
	end
	say

return key1

/* Common error text to print to screen */
PrintError: procedure expose fail

	parse ARG rc

	say 'There was an error. This process has not been completed!'
	say 'The error code returned was 'rc'.'
	say 'Please re-run this option when the problem is fixed.'
	call Hit_Key 'M'
	say
	fail = 1

return

/* procedure view log */
view_log: procedure expose burnlog viewer

	if (stream(burnlog, 'C', 'query exists') == '') then do
		say burnlog' does not exist!'
		say
		say 'A process has not yet been run or an error has occurred while trying'
		say 'to write to 'burnlog'.'
		say
		call Hit_Key 'M'
	end
	else '@start "BurnLog" /F 'viewer burnlog

return

/* procedure build boot image */
make_bootimage:

	parse arg Batch_Mode

	call chdir source /* ensure we are in the source directory */
	say 'Making boot image, please wait!'
	if (stream('DISK_*.IMG', 'C', 'query exists') \= '') then do
		if (Batch_Mode == "NO") then do
			say 'Boot image files already exist. Overwrite? (Y/N)'
			if (GetKeyYN(k) == 'N') then do
				flag2 = '*'
				return
			end
		end
		call SysFileDelete('DISK_0_1.IMG')
		call SysFileDelete('DISK_2.IMG')
	end
             
	'@echo Starting to make bootimage files ready for burning > 'burnlog

	rcode = ''
	say 'Preparing 1st boot image, please wait!'
	call size_floppy '2.88'  /* size the virtual floppy to 2.88MB */
	call chdir source /* ensure we are in the source directory */
	call chdir 'UpdCD\bootimg\DISK_0_1.IMG'
	'@SYSINSTX 'svdc_drive' >>'burnlog' 2>>&1'
	rcode = rcode' 'rc
	'@XCOPY * 'svdc_drive' /H/O/T/R/V >>'burnlog' 2>>&1'
	rcode = rcode' 'rc
	call chdir rootdir /* ensure we are in updcd directory. */
	say 'Modifying boot record...'
	'@cdboot.exe /drive 'substr(svdc_drive, 1, 1)' /timeout 10 >> 'burnlog' 2>>&1'
	rcode = rcode' 'rc
	call chdir source /* ensure we are in the source directory */
	call chdir 'diskimgs'
	'@IMAGE 'svdc_drive' ..\DISK_0_1.IMG >>'burnlog' 2>>&1'
	rcode = rcode' 'rc

	say 'Preparing 2nd boot image, please wait!'
	call size_floppy '1.44'  /* size the virtual floppy to 1.44MB */
	call chdir source /* ensure we are in the source directory */
	call chdir 'UpdCD\bootimg\DISK_2.IMG'
	'@XCOPY * 'svdc_drive' /H/O/T/R/V >>'burnlog' 2>>&1'
	rcode = rcode' 'rc
	call chdir source /* ensure we are in the source directory */
	call chdir 'diskimgs'
	'@label 'svdc_drive'DISK 2 >>'burnlog' 2>>&1'
	rcode = rcode' 'rc
	'@IMAGE 'svdc_drive' ..\DISK_2.IMG >>'burnlog' 2>>&1'
	rcode = rcode' 'rc

	call size_floppy '2.88'  /* size the virtual floppy to 2.88MB */
	'@echo Ended with return string: 'rcode' >> 'burnlog
	error = 0
	do while length(rcode) > 0
		parse var rcode rcd rcode
		if rcd <> 0 then error = 1
	end
	if error = 0 then do
		flag2 = '*'
		say 'Completed without errors.'
	end
	if (Batch_Mode == 'NO') then call Hit_Key 'M'

return

/* procedure build CD-ROM image */
make_cdimage: 

	parse arg Batch_Mode

	/* check RSJ */
	if bprog = 'RSJ' then do
		flag3 = '*'
		say 'You are not supposed to run this option if you use RSJ.'
		say
		if (Batch_Mode == 'NO') then call Hit_Key 'M'
		return
	end

	/* check mkisofs */
	trace -1
	'@'cdrdir'\mkisofs -version >nul 2>>&1'
	if (rc <> 0) then do
		say
		say 'MkISOFS is not correctly installed.'
		say 'Please install it before using this option.'
		call Hit_Key 'C'
		return
	end

	/* make updated CD imagefile */
	say 'Making CD image using 'source' (source) and 'tempraw' (target).'
	call SysFileTree tempdir, stem, 'D'
	if (stem.0 < 1) then call SysMkDir tempdir
	if (stream(tempraw, 'C', 'query exists') \= '') then do
		if (Batch_Mode == 'NO') then do
			say '['tempraw'] already exists. Overwrite? (Y/N)'
			if (GetKeyYN(k) == 'N') then do
				flag3 = '*'
				return
			end
		end
		call SysFileDelete(tempraw)
	end

	if (stream(source'\BOOTIMGS\*.IMG', 'C', 'query exists') \= '') then
		boot = '-bBOOTIMGS/disk_0_1.img -cboot.cat'

	call chdir source /* ensure we are in the source directory */
	say
	say 'This process is currently being run and may take a few minutes to'
	say 'complete. If no disk activity occurs for several seconds, press CTRL-C'
	say 'to exit this script.'
	if substr(source, 2, 1) = ':' & length(source) = 2 then 
		src = source||'/' 
	else 
		src = source
	'@'cdrdir'\mkisofs -f -v -J -D -U -o 'tempraw boot src' >>'burnlog' 2>>&1'
	
	if (rc == 0) then flag3 = '*'
	say
	say 'Completed with return code 'rc
	if (Batch_Mode == 'NO') then call Hit_Key 'M'

return

/* procedure blank CD-RW */
blank_cd:

	parse arg Batch_Mode

	if bprog = 'RSJ' then do
		if (rsjdir == '') then do
			say 'Please install RSJ before using this option.'
			call Hit_Key 'M'
			return
		end
		if (\attached) then
			if (\attachDev()) then do
				fail = 1 /* failure */
				return
			end
	end
	else do
		/* check cdrecord */
		trace -1
		'@'cdrdir'\cdrecord -version >nul 2>>&1'
		if (rc <> 0) then do
			say
			say 'CDRecord/2 is not correctly installed.'
			say 'Please install it before using this option.'
			call Hit_Key 'C'
			return
		end
	end

	/* erase CDRW */
	if (Batch_Mode == 'NO') then do
		say 'Shall I blank the CD-RW in your burner? (Y/N)'
		if (GetKeyYN(k) == 'N') then do
			call Hit_Key 'M'
			return
		end
	end

	say 'Blanking CD-RW media now. Please wait.'
	if bprog = 'CDR' then
			'@'cdrdir'\cdrecord dev='device' speed='speed' blank=fast >>'burnlog' 2>>&1'
	else do
			cmd = rsjdir'\cdwfmt '||CD||' /ERASE'
			'@echo '||cmd||' >> '||burnlog
			'@'||cmd||' >>'||burnlog||' 2>>&1'
	end

	if (rc \= 0) then do
		call PrintError rc
		return
	end
	else do
		say 'Done!'
		if (rsjdir \= '') then 
			if (\detach()) then iterate
	flag4 = '*'

		say 'Completed with return code 'rc
		if (Batch_Mode == 'NO') then call Hit_Key 'M'
	end

return

/* procedure burn CD-ROM */
burn_cd:

	parse arg Batch_Mode

	if (Batch_Mode == 'NO') then do
		say 'Is a blank CD-R or CD-RW in your burner? (Y/N)'
		if (GetKeyYN(k) == 'N') then do
			say
			say 'Please insert a blank CD-R or CD-RW into the CD burner.'
			say 'Type Y to continue or N to return to the main menu.'
			if (GetKeyYN(k) == 'N') then return
		end
	end

	/* burn CD */
	if bprog = 'CDR' then do
		/* check if cdrecord installed */
		trace -1
		'@'cdrdir'\cdrecord -version >nul 2>>&1'
		if (rc <> 0) then do
			say
			say 'CDRecord/2 is not correctly installed.'
			say 'Please install it before using this option.'
			call Hit_Key 'C'
			return
		end
		isize = stream(tempraw, 'C', 'query size')
		if (isize == '') then do
			say 'The imagefile ['tempraw'] does not exist. Please make the'
			say 'imagefile first via process #3.'
			say
			call Hit_Key 'M'
			return
		end
		else do
			isizeK = isize / 1024
			isizeM = isize / 1048576
			itimeMIN = isize / (153600 * speed) % 60
			itimeSEC = isize / (153600 * speed) // 60

			say 'About to burn updated CD-ROM using imagefile ['tempraw'].'
			say
			say 'The imagefile size in kilobytes is: 'isizeK
			say 'The imagefile size in megabytes is: 'format(isizeM,,2)

			/* check size */
			if (isizeM > 650) then do
				say
				say 'Warning: data may not fit on current disk.'
				say 'You may still be able to burn the CD using the overburn option.'
				say 'Continue? (Y/N)'
				if (GetKeyYN(k) == 'N') then return
			end

			say
		end
		say 'Burner device: 'device', speed: 'speed '('150*speed' KB/s)'
		say 'This burn will complete in approximately 'itimeMIN':'format(itimeSEC,,2)' minutes.'
		say 'The time now is 'TIME('C')
		say
		say 'Writing updated installation CD-ROM. Please wait.'
		'@'cdrdir'\cdrecord dev='device' speed='speed' -v -eject -pad -data 'extrap tempraw' >>'burnlog' 2>>&1'

		if (rc \= 0) then do
			call PrintError rc
			return
		end
		else flag5 = "*"
	end
	else do

		/* check software */
		if (rsjdir == '') then do
			say 'Please install RSJ before using this option.'
			call Hit_Key 'M'
			return
		end

		/* check size */
		say 'Checking image...'
		call chdir source /* ensure we are in the source directory */
		call SysFileTree '*', 'junk.', 'FS'
		tsize = 0
		do i = 1 to junk.0
			parse var junk.i . . size .
			tsize=tsize+size
		end
		tsize = format(tsize/1048576,,2)
		say 'The estimated image size in megabytes is: 'tsize
		if (tsize > 650) then do
			say
			say 'Warning: data may not fit on current disk.'
			say 'You may still be able to burn the CD using the overburn option.'
			say 'Continue?'
			if (GetKeyYN(k) == 'N') then return
		end

		if (\attached) then
			if (\attachDev()) then do
				fail = 1
				return
			end
			rc = 0
			call charout, 'Burning updated installation files to CD...'
		cmd = rsjdir||'\cdcopy '||source||'\*.* '||CD||'\ /s'
		'@echo '||cmd||' >> '||burnlog
		'@'||cmd||' >>'||burnlog||' 2>>&1'
		if (rc == 0) then do
				say 'Done!'
				if (stream(source'\BOOTIMGS\DISK_0_1.IMG', 'C', 'query exists') \= '') then do
					call charout, 'Found bootable image files. Burning boot sector data to CD...'
					/* now make image file bootable */
					cmd = rsjdir||'\cdboot.exe '||CD||'\BOOTIMGS\DISK_0_1.IMG -s '||source||'\BOOTIMGS\DISK_0_1.IMG'
					'@echo '||cmd||' >> '||burnlog
					'@'||cmd||' >>'||burnlog||' 2>>&1'
					if (rc \= 0) then do
						say 'Error!'
						say 'Boot sector data was not transferred to CD.'
						call PrintError rc
						return
					end
					else say 'Done!'
				end
				if (detach()) then flag5 = '*'
			end
			else do
				say 'Error!'
				call PrintError rc
				fail = 1
				return
			end
		end

	say 'Completed with return code 'rc
	if (Batch_Mode == 'NO') then call Hit_Key 'M'

return

/* expand or contract virtual disk drive. */
size_floppy:

	parse arg floppy_size

	currentdir = chdir() /* save the current directory. */
	call chdir rootdir /* ensure we are in updcd directory. */

	'@echo Trying to insert a 'floppy_size'MB virtual disk in drive 'svdc_drive' >> 'burnlog
	if (vdisk == 'VFDISK') then do
		if (floppy_size == '2.88') then vfctrl_size = 4
		else vfctrl_size = 1
		'@vfctrl 'svdc_drive                ' 0 >>'burnlog' 2>>&1'
		'@vfctrl 'svdc_drive' 'vfctrl_size'     >>'burnlog' 2>>&1'
	end
	else do
		'@SVDC /ef '             svdc_drive' 		>>'burnlog' 2>>&1'
		'@SVDC /r:'floppy_size' 'svdc_drive' 		>>'burnlog' 2>>&1'
		'@SVDC /i:'floppy_size' 'svdc_drive' 		>>'burnlog' 2>>&1'
	end

	if (rc <> 0) then do
		say 'Error inserting a 'floppy_size'MB virtual floppy in drive 'svdc_drive
		exit 9
	end

	parse value SysDriveInfo(svdc_drive) with . free .
	'@echo Detected 'free' bytes free space on drive 'svdc_drive' >> 'burnlog
	call chdir currentdir /* return to original directory */

return

/* get response */
get_response:

	say 'Specify the steps separated by spaces (e.g. 2 3 5):'
	pull response
	automated = 1
	In_Batch_Mode = 'YES'
	flag6 = '*'
	if (rsjdir \= '') then
		if ((pos( 5, response ) > 0) & (\attached)) then
			if (\attachDev()) then return
	fail = 0

return

/* attach CD-ROM */
detach: procedure expose CD burnlog attached rsjdir

	success = 1
	cmd = rsjdir'\cdattach '||CD||' -s'
	call charout, 'Finalizing session...'
	'@echo '||cmd||' >> '||burnlog
	'@'||cmd||' >>'||burnlog||' 2>>&1'

	if (rc \= 0) then do
		say 'Error!'
		success = 0
		call PrintError rc
	end
	else do
		say 'Done!'
		attached = 0
	end

return success

/* get CD drive letter */
attachDev: procedure expose burnlog CD attached rsjdir

	if (CD == '') then do
		tmp = SysIni('USER', 'CDWFS', 'DefaultDriveLetter')
		if ((tmp = '') | (tmp = 'ERROR:')) then tmp = 'Z:'
		else tmp = strip(tmp, 'T', '00'x)
		call charout, 'Type the drive letter you wish to attach to your burner (default '||tmp||').'
		pull junk
		if (junk \= '') then CD = substr(junk, 1, 1 )||':'
		else CD = tmp
	end

	cmd = rsjdir'\cdattach.exe '||CD

	do forever
		rc = 0
		call charout, 'Attaching CD...'
		'@echo '||cmd||' >> '||burnlog
		'@'||cmd||' >>'||burnlog||' 2>>&1'
		select
			when (rc == 0) then do
				attached = 1
				say 'Done!'
				leave
			end
			when (rc == 21) then do
				say 'Error!'
				say 'Could not attach media. Check to make sure media is in drive.'
				call charout, 'Would you like me to try again?'
				if (GetKeyYN(k) == 'N') then leave
			end
			when (rc == 85) then do
				say 'Error!'
				call charout, 'Warning! Media is already attached. Continue anyway?'
				if (GetKeyYN(k) == 'N') then attached = 1
				leave
			end
			otherwise do
				say 'Unknown Error! The device could not be attached.'
				say 'This procedure will now end.'
				call Hit_Key 'C'
				leave
			end
		end  /* Select */
	end /* Do forever */

return attached

/* hit key */
Hit_Key: procedure

	parse arg option

	if (option == 'C') then prompt = 'continue.'
	else prompt = 'return to the menu.'

	say 'Hit any Key to 'prompt
	call SysGetKey 'NOECHO'

return

/* read the updcd cfg file and get absolute directory path */
Read_Config_File: 

	parse arg cfgfile

	/* check */
	if stream(cfgfile, 'c', 'query exists') = '' then do
		say 'Fatal error! The configuration file ('cfgfile') is missing.'
		say 'Please run INSTALL.CMD to create one.'
		'@pause'
		exit 9
	end

	/* read file */
	do while lines(cfgfile)
		l = linein(cfgfile)
		interpret l
	end
	call lineout cfgfile 

	/* need to know abolute path */
	cdir    = chdir()
	burn    = chdir(burn)
	cdir    = chdir(cdir)
	w4cd		= chdir(w4cd)
	cdir    = chdir(cdir)
	fixes   = chdir(fixes)
	cdir    = chdir(cdir)
	updates = chdir(updates)
	cdir    = chdir(cdir)
	addons  = chdir(addons)
	cdir    = chdir(cdir)
	rootdir = chdir(rootdir)
	cdir    = chdir(cdir)
	tempdir = chdir(tempdir)
	cdir    = chdir(cdir)
	source	= chdir(source)
	cdir    = chdir(cdir)
	cdrdir	= chdir(cdrdir)
	cdir    = chdir(cdir)

	/* need to know abolute path for files */
	if filespec('drive', log) = '' then 
		log	= cdir||'\'||filespec('name', log)
	if filespec('drive', tempraw) = '' then 
		tempraw	= cdir||'\'||filespec('name', tempraw)
	if filespec('drive', burnlog) = '' then 
		burnlog	= cdir||'\'||filespec('name', burnlog)

return

/* chdir: drive and path aware directory() replacement       */
/*        returns the current dir without trailing backslash */
chdir: procedure
	parse upper arg dr                /* get target dir        */
	if length(dr) > 1 then dr = strip(dr, 'T', '\')
	cdir = directory()                /* get current dir       */
	if substr(dr, 2, 1) = ':' then do /* absolute path         */
		drive = substr(dr, 1, 2)
		rc = directory(drive)           /* change to drive       */
		if substr(rc, 1, 2) <> drive then 
			return strip(directory(cdir), 'T', '\')
		rc = directory('\')             /* change to root        */
		rc = directory(substr(dr, 3))   /* change to path        */
		if strip(rc, 'T', '\') <> dr then 
			return strip(directory(cdir), 'T', '\')
	end
	else do
		return strip(directory(dr), 'T', '\')  /* relative path  */
	end
return strip(directory(), 'T', '\') /* return current dir    */

